import { parse } from "https://deno.land/std@0.106.0/flags/mod.ts";
import { createCanvas } from "https://deno.land/x/canvas@v1.3.0/mod.ts";
import { CollageModeType, collageModeType } from "../src/common/collages.ts";
import DenoCollageModes, {
  CastCanvasRenderingContext,
  ProxyImage,
} from "./collages.ts";
import { CollageConfig } from "../src/common/types.ts";
import { choice, shuffle } from "../src/common/utils.ts";
import { walkSync } from "https://deno.land/std@0.107.0/fs/mod.ts";

const args = parse(Deno.args, {
  alias: {
    "w": "width",
    "h": "height",
    "o": "output",
    "m": "mode",
    "r": "recursive",
  },
  boolean: ["recursive"]
});

if (args["mode"] === true) {
  console.log(collageModeType.join(", "));
  Deno.exit(0);
}

const files: Set<string> = new Set();
const includeExtensions = Array.from(
  String(args["include"] || "*.png, *.jpg").matchAll(/\*\.([\w]+)/g),
).map(([_, group]) => group);

args["_"].forEach((arg) => {
  arg = arg.toString();
  if (Deno.statSync(arg).isDirectory) {
    Array.from(
      walkSync(arg, {
        maxDepth: args["recursive"]
          ? Infinity
          : 1,
        includeDirs: false,
        includeFiles: true,
        exts: includeExtensions.length ? includeExtensions : undefined,
      }),
    ).forEach((entry) => files.add(entry.path));
  } else {
    files.add(arg);
  }
});

if (files.size < 2) {
  console.error("kollagen needs at least 2 images to work.");
  Deno.exit(1);
}

const shuffledFiles = shuffle(Array.from(files));

const images: ProxyImage[] = shuffledFiles.map((file) => new ProxyImage(file));

const modes = new DenoCollageModes();

const modeKey: CollageModeType = args["mode"] || choice(collageModeType);
const mode = modes.modes[modeKey];

console.log(
  `Creating a "${mode.name}" collage, choosing from ${shuffledFiles.length} files...`,
);
// console.debug(`Images: ${shuffledFiles.join(", ")}`);

const canvas = createCanvas(args["width"] || 640, args["height"] || 640);
const context = canvas.getContext("2d");

const collageConfig: CollageConfig = {
  numImages: args["n"],
};

const segments = mode.getSegments(
  context as CastCanvasRenderingContext,
  collageConfig,
  images,
);
mode.place(context as CastCanvasRenderingContext, images, segments);

const output = args["output"] || "collage.png";
console.log(`Saving to "${output}"...`);
await Deno.writeFile(output, canvas.toBuffer());
